package com.jpsoft.smart.lapi.handler;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jpsoft.smart.config.OSSConfig;
import com.jpsoft.smart.lapi.dto.PersonVerification;
import com.jpsoft.smart.lapi.dto.PersonVerificationResponseInfo;
import com.jpsoft.smart.modules.base.entity.PersonInfo;
import com.jpsoft.smart.modules.base.entity.ProjectInfo;
import com.jpsoft.smart.modules.base.service.PersonDeviceFilterLogService;
import com.jpsoft.smart.modules.base.service.PersonDeviceLogService;
import com.jpsoft.smart.modules.base.service.PersonInfoService;
import com.jpsoft.smart.modules.base.service.ProjectInfoService;
import com.jpsoft.smart.modules.business.service.WorkAttendanceService;
import com.jpsoft.smart.modules.common.utils.AESUtil;
import com.jpsoft.smart.modules.common.utils.LApiUtil;
import com.jpsoft.smart.modules.common.utils.OSSUtil;
import com.jpsoft.smart.modules.realname.dto.WorkerAttendanceDTO;
import com.jpsoft.smart.modules.sys.entity.SysLog;
import com.jpsoft.smart.modules.sys.service.SysLogService;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.codec.http.*;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.SocketAddress;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

@Slf4j
@Component
@ChannelHandler.Sharable
public class PersonVerificationHandler extends SimpleChannelInboundHandler<PersonVerification> {
    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private PersonDeviceLogService personDeviceLogService;

    @Autowired
    private PersonDeviceFilterLogService personDeviceFilterLogService;

    @Autowired
    private WorkAttendanceService workAttendanceService;

    @Autowired
    private SysLogService sysLogService;

    @Autowired
    private ProjectInfoService projectInfoService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    RabbitTemplate rabbitTemplate;

    private  final String X_FRAME_OPTIONS = "X-Frame-Options";
    private  final String SAMEORIGIN = "SAMEORIGIN";
    private  final String JSON_PLAIN = "application/json";
    private  final String CONTENT_LENGTH = "Content-Length";
    private  final String CONTENT_TYPE = "Content-Type";
    private  final String CONNECTION = "Connection";

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, PersonVerification personVerification) throws Exception {
        new Thread(()->{
            asyncWork(ctx,personVerification);
        }).start();
    }

    public void asyncWork(ChannelHandlerContext ctx,PersonVerification personVerification) {
        log.warn("===人员记录异步处理 begin===");

        JSONObject jsonObject = personVerification.getJsonObject();

        JSONObject faceJson = new JSONObject(jsonObject.getStr("FaceInfoList").substring(1,jsonObject.getStr("FaceInfoList").length()-1));

        Long timestamp = faceJson.getLong("Timestamp");

        if (timestamp.toString().length()<13){
            double timestampDouble = timestamp *  Math.pow(10, 13 - timestamp.toString().length());
            timestamp =new Double(timestampDouble).longValue() ;
        }

        //log.warn(String.valueOf(timestamp));
        Date date = new Date(timestamp);

        //设备序列号
        String deviceNo = jsonObject.getStr("DeviceCode");

        //人员温度信息
        BigDecimal temperature = faceJson.getBigDecimal("Temperature");

        //戴口罩状态（0：未开启口罩验证，1：未戴口罩，2：戴口罩）
        Integer maskFlag = faceJson.getInt("MaskFlag");

        //人员抓拍信息
        JSONObject faceImageJson = faceJson.getJSONObject("FaceImage");

        //匹配状态信息
        JSONObject libMatInfoListJson = jsonObject.getJSONArray("LibMatInfoList").getJSONObject(0);

        if(libMatInfoListJson==null){
            libMatInfoListJson = new JSONObject();
        }

        //匹配人员信息
        JSONObject matchPersonInfo = new JSONObject();
        if (StringUtils.isNotBlank(libMatInfoListJson.getStr("MatchPersonInfo"))){
            matchPersonInfo = libMatInfoListJson.getJSONObject("MatchPersonInfo");
        }

        log.warn("人员记录写入");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        HashMap map = new HashMap<>();
        map.put("deviceNo",deviceNo);
        map.put("temperature",temperature);
        map.put("libMatInfoListJson",libMatInfoListJson);
        map.put("matchPersonInfo",matchPersonInfo);
        map.put("recordTime",sdf.format(date));

        log.warn(map.toString());

        SysLog sysLog = new SysLog();
        sysLog.setPointcut(deviceNo);
        sysLog.setUrl("/LAPI/V1.0/System/Event/Notification/PersonVerification");
        sysLog.setData(map.toString());
        sysLog.setRemark("收到人脸识别信息");
        SocketAddress remoteAddress = ctx.channel().remoteAddress();
        sysLog.setRemoteIp(remoteAddress.toString());
        sysLog.setCreateTime(new Date());
        sysLogService.insert(sysLog);

        //上传人脸信息
        String faceImageUrl = "";

        try {
            if (StringUtils.isNotBlank(faceImageJson.getStr("Data"))){
                BASE64Decoder decoder = new BASE64Decoder();

                byte[] imgData = decoder.decodeBuffer(faceImageJson.getStr("Data"));

                for (int i = 0; i < imgData.length; ++i) {
                    if (imgData[i] < 0) {// 调整异常数据
                        imgData[i] += 256;
                    }
                }

                ByteArrayInputStream inputStream = new ByteArrayInputStream(imgData);


                faceImageUrl = OSSUtil.upload(ossConfig, "/devicePersonLog", faceImageJson.getStr("Name"), inputStream);
            }

        } catch (Exception e) {
            log.error("保存图片路径错误" + e.getMessage());
        }

        //记录人员记录
        personDeviceLogService.deviceInsertLog(deviceNo,temperature,faceImageJson,libMatInfoListJson,matchPersonInfo,date,maskFlag,faceImageUrl);

        //记录人员过滤记录
        personDeviceFilterLogService.deviceInsertFilterLog(deviceNo,temperature,faceImageJson,libMatInfoListJson,matchPersonInfo,date,maskFlag,faceImageUrl);

        //记录考勤信息
        try {
            final PersonInfo personInfo = LApiUtil.getPersonCode(libMatInfoListJson, matchPersonInfo);
            final BigDecimal fTemperature = temperature;
            final Date recordDate = date;

            if(personInfo.getId()!=null && personInfo.getId()!=0L) {
                workAttendanceService.punchIn(personInfo.getId(), fTemperature, recordDate);

                PersonInfo destPerson = personInfoService.get(personInfo.getId());

                if(destPerson!=null) {
                    //查询是否关联了项目，如果是则需要上传到第三方服务器
                    ProjectInfo projectInfo = projectInfoService.findOneByCompanyId(destPerson.getCompanyId());

                    if (StringUtils.isNotEmpty(destPerson.getIdCard())
                            && projectInfo != null
                            && StringUtils.isNotEmpty(projectInfo.getAppId())
                            && StringUtils.isNotEmpty(projectInfo.getAppSecret())) {
                        WorkerAttendanceDTO dto = new WorkerAttendanceDTO();

                        dto.setProjectCode(projectInfo.getProjectCode());
                        dto.setIdNumber(destPerson.getIdCard());
                        dto.setSocialCreditCode(projectInfo.getSocialCreditCode());
                        dto.setStartTime(new DateTime(date).toString("yyy-MM-dd HH:mm:ss"));
                        dto.setSignDate(new DateTime(date).toString("yyy-MM-dd"));

                        rabbitTemplate.convertAndSend("importImageExchange", "syncWorkerAttendence", dto);
                    }
                }
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
        }

        log.warn("===人员记录异步处理 end===");

        //返回数据
        try {
            log.warn("===人员记录回包处理 start===");
            PersonVerificationResponseInfo responseInfo = new PersonVerificationResponseInfo();

            responseInfo.getResponse().setResponseURL("/LAPI/V1.0/PACS/Controller/Event/Notifications");
            responseInfo.getResponse().setStatusCode(0);
            responseInfo.getResponse().setStatusString("Succeed");
            responseInfo.getResponse().getData().setRecordID(jsonObject.getInt("ID"));
            responseInfo.getResponse().getData().setTime(DateTime.now().toString("yyyy-MM-dd HH:mm:ss"));

            String json = JSONUtil.toJsonStr(responseInfo);

            FullHttpResponse response = new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.OK, Unpooled.wrappedBuffer(json.getBytes("UTF-8")));
            response.headers().set(CONTENT_LENGTH, response.content().readableBytes());
            response.headers().set(CONTENT_TYPE, JSON_PLAIN);
            response.headers().set(CONNECTION, HttpHeaderValues.CLOSE);
            response.headers().set(X_FRAME_OPTIONS, SAMEORIGIN);

            ctx.channel().writeAndFlush(response);

            log.warn("===人员记录回包结束 end===");
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
        }
    }
}
