package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.entity.CompanyDeviceRelation;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.service.CompanyDeviceRelationService;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import com.jpsoft.smart.modules.common.utils.HttpUtil;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.base.entity.DeviceInfo;
import com.jpsoft.smart.modules.base.service.DeviceInfoService;
import com.jpsoft.smart.modules.lapi.channel.LapiChannel;
import com.jpsoft.smart.modules.lapi.service.ILapiService;
import com.jpsoft.smart.modules.lapi.utils.LapiSessionUtil;
import com.jpsoft.smart.modules.sys.entity.SysLog;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.sys.service.DataDictionaryService;
import com.jpsoft.smart.modules.sys.service.SysLogService;
import com.jpsoft.smart.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;

@RestController
@RequestMapping("/base/deviceInfo")
@Api(description = "设备信息")
public class DeviceInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private DeviceInfoService deviceInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private ILapiService lapiService;

    @Autowired
    private UserService userService;

    @Autowired
    private CompanyDeviceRelationService companyDeviceRelationService;

    @Autowired
    private SysLogService sysLogService;

    @Autowired
    private CompanyInfoService companyInfoService;



    @ApiOperation(value="添加设备")
    @Transactional(rollbackFor = Exception.class)
    @PostMapping("add")
    public MessageResult<DeviceInfo> add(@RequestBody DeviceInfo deviceInfo,@RequestAttribute String subject){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        try {
            DeviceInfo item = deviceInfoService.getByDeviceNo(deviceInfo.getDeviceNo());

            if(item == null) {

                deviceInfo.setId(UUID.randomUUID().toString());
                deviceInfo.setDelFlag(false);
                deviceInfo.setCreateBy(subject);
                deviceInfo.setCreateTime(new Date());

                int affectCount = deviceInfoService.insert(deviceInfo);

                /*
                //设备添加人脸库
                List<LapiResult> lapiResults= lapiService.addCompanyLib(deviceInfo.getCompanyId(),deviceInfo.getId());
                for (int i = 0; i < lapiResults.size(); i++) {
                    LapiResult lapiResult = lapiResults.get(i);

                    if (lapiResult.isSuccess()) {
                        //默认人脸库ID defaultFaceLibraryId
                        //LibraryId
                        //LibraryId -> 1585025356
                        HashMap dataMap = (HashMap) lapiResult.getData();
                        String libraryId = dataMap.get("LibraryId").toString();
                        System.out.println("LibraryId=============" + libraryId);

                        //添加信息到设备与公司关联表
                        //先查关联表是否存在数据
                        CompanyDeviceRelation companyDeviceRelation = companyDeviceRelationService.findByCompanyIdAndDeviceId(deviceInfo.getCompanyId(),deviceInfo.getId());
                        if(companyDeviceRelation == null ){
                            companyDeviceRelation = new CompanyDeviceRelation();
                            companyDeviceRelation.setId(UUID.randomUUID().toString());
                        }
                        companyDeviceRelation.setCompanyId(deviceInfo.getCompanyId());
                        companyDeviceRelation.setDeviceId(deviceInfo.getId());
                        //保存人脸库ID
                        companyDeviceRelation.setLibraryId(libraryId);
                        companyDeviceRelationService.insert(companyDeviceRelation);
                    }
                }
                */

                //添加信息到设备与公司关联表
                //先查关联表是否存在数据
                CompanyDeviceRelation companyDeviceRelation = companyDeviceRelationService.findByCompanyIdAndDeviceId(deviceInfo.getCompanyId(),deviceInfo.getId());
                if(companyDeviceRelation == null ){
                    companyDeviceRelation = new CompanyDeviceRelation();
                    companyDeviceRelation.setId(UUID.randomUUID().toString());
                }
                companyDeviceRelation.setCompanyId(deviceInfo.getCompanyId());
                companyDeviceRelation.setDeviceId(deviceInfo.getId());
                //保存人脸库ID
                companyDeviceRelation.setLibraryId("");
                companyDeviceRelationService.insert(companyDeviceRelation);


                if (affectCount > 0) {
                    msgResult.setResult(true);
                    msgResult.setData(deviceInfo);
                } else {
                    msgResult.setResult(false);
                    msgResult.setMessage("数据库添加失败");
                }
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("设备编号已经存在，不能重复添加！");
            }
        }
        catch(Exception ex){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<DeviceInfo> create(){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        DeviceInfo deviceInfo = new DeviceInfo();

        msgResult.setData(deviceInfo);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="获取设备信息")
    @GetMapping("edit/{id}")
    public MessageResult<DeviceInfo> edit(@PathVariable("id") String id){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        try {
            DeviceInfo deviceInfo = deviceInfoService.get(id);

            if (deviceInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(deviceInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新设备信息")
    @PostMapping("update")
    public MessageResult<DeviceInfo> update(@RequestBody DeviceInfo deviceInfo,@RequestAttribute String subject){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();

        try {
            DeviceInfo item = deviceInfoService.getByDeviceNo(deviceInfo.getDeviceNo());

            if(item!=null) {
                //如果修改自己的属性
                if (item.getId().equals(deviceInfo.getId())) {
                    deviceInfo.setUpdateBy(subject);
                    deviceInfo.setUpdateTime(new Date());

                    int affectCount = deviceInfoService.update(deviceInfo);

                    if (affectCount > 0) {
                        //判断是否修改公司
                        //不相同为修改
                        if(!item.getCompanyId().equals(deviceInfo.getCompanyId())) {
                            //添加信息到设备与公司关联表
                            //先删除
                            companyDeviceRelationService.deleteByCompanyIdAndDeviceId(item.getCompanyId(), item.getId());

                            CompanyDeviceRelation companyDeviceRelation = new CompanyDeviceRelation();
                            companyDeviceRelation.setId(UUID.randomUUID().toString());
                            companyDeviceRelation.setCompanyId(deviceInfo.getCompanyId());
                            companyDeviceRelation.setDeviceId(deviceInfo.getId());
                            //保存人脸库ID
                            companyDeviceRelation.setLibraryId("");
                            companyDeviceRelationService.insert(companyDeviceRelation);
                        }
                        msgResult.setResult(true);
                        msgResult.setData(deviceInfo);
                    } else {
                        msgResult.setResult(false);
                        msgResult.setMessage("数据库更新失败");
                    }
                } else {
                    //如果修改设备编号在数据库中已有重复的设备号，则不能修改！
                    msgResult.setResult(false);
                    msgResult.setMessage("该设备编号的设备已存在，不能修改！");
                }
            }
            else{
                msgResult.setResult(false);
                msgResult.setMessage("该设备不存在！");
            }

        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除设备")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            DeviceInfo deviceInfo = deviceInfoService.get(id);
//            //删除所有设备人脸库
              //清除人脸数据
              //危险操作
//            List<LapiResult> lapiResults= lapiService.deleteDeviceAllCompanyLib(id);
//            for (int i = 0; i < lapiResults.size(); i++) {
//                LapiResult lapiResult = lapiResults.get(i);
//                if (lapiResult.isSuccess()) {
//                    Object obj = lapiResult.getData();
//                    //默认人脸库ID defaultFaceLibraryId
//                    //LibraryId
//                    //LibraryId -> 1585025356
//
//                    //设备上的人脸库删除成功后
//                    //执行删除设备
//                    deviceInfo.setDelFlag(true);
//                    deviceInfo.setUpdateBy(subject);
//                    deviceInfo.setUpdateTime(new Date());
//                    affectCount = deviceInfoService.update(deviceInfo);
//                }
//            }

            deviceInfo.setDelFlag(true);
            deviceInfo.setUpdateBy(subject);
            deviceInfo.setUpdateTime(new Date());
            int affectCount = deviceInfoService.update(deviceInfo);

            if (affectCount > 0) {
                companyDeviceRelationService.deleteByCompanyIdAndDeviceId(deviceInfo.getCompanyId(),deviceInfo.getId());
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除设备")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                DeviceInfo deviceInfo = deviceInfoService.get(id);
                deviceInfo.setDelFlag(true);
                deviceInfo.setUpdateBy(subject);
                deviceInfo.setUpdateTime(new Date());

                affectCount += deviceInfoService.update(deviceInfo);

                companyDeviceRelationService.deleteByCompanyIdAndDeviceId(deviceInfo.getCompanyId(),deviceInfo.getId());
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="设备列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "aliasName",value = "设备别名",  paramType = "query"),
            @ApiImplicitParam(name = "deviceNo",value = "编号",  paramType = "query"),
            @ApiImplicitParam(name = "address",value = "位置",  paramType = "query"),
            @ApiImplicitParam(name = "companyId",value = "企业ID",  paramType = "query")
    })
    public MessageResult<Map> pageList(
            String aliasName,String deviceNo,String address,String companyId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.is_online","desc"));
        sortList.add(new Sort("a.create_time","desc"));

        //为公司管理员
        boolean is_admin = userService.hasRole(subject,"ADMIN");
        if (is_admin){
            User user = userService.get(subject);
            String userCompanyId = user.getCompanyId();

            List<DeviceInfo> deviceInfoList = deviceInfoService.findByCompanyId(userCompanyId);
            List<String> deviceIdList = new ArrayList<String>();
            if(deviceInfoList.size() > 0) {
                for (DeviceInfo deviceInfo : deviceInfoList) {
                    deviceIdList.add(deviceInfo.getId());
                }
            }else{
                deviceIdList.add("-1");
            }

            searchParams.put("deviceIdList",deviceIdList);
        }


        if (StringUtils.isNotEmpty(aliasName)) {
            searchParams.put("aliasName","%" + aliasName + "%");
        }
        if (StringUtils.isNotEmpty(deviceNo)) {
            searchParams.put("deviceNo","%" + deviceNo + "%");
        }
        if (StringUtils.isNotEmpty(address)) {
            searchParams.put("address","%" + address + "%");
        }
        if (StringUtils.isNotEmpty(companyId)) {
            searchParams.put("companyId",companyId);
        }
        Page<DeviceInfo> page = deviceInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (DeviceInfo deviceInfo:page) {
            String typeN = dataDictionaryService.findNameByCatalogNameAndValue("设备类型",deviceInfo.getType());
            deviceInfo.setTypeN(typeN);
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="根据IP地址和端口号查找设备")
    @PostMapping("getByIpAddressAndPort")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ipAddress",value = "IP地址", paramType = "query"),
            @ApiImplicitParam(name = "port",value = "端口号", paramType = "query")
    })
    public MessageResult<DeviceInfo> getByIpAddressAndPort( String ipAddress,String port,@RequestAttribute String subject) {
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();
        try {
            DeviceInfo device = lapiService.findByIpAddressAndPort(ipAddress, port);

            msgResult.setResult(true);
            msgResult.setData(device);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }
        return msgResult;
    }


    @ApiOperation(value="获取设备信息列表")
    @PostMapping("list")
    public MessageResult<List<DeviceInfo>> list(@RequestAttribute String subject) {
        MessageResult<List<DeviceInfo>> msgResult = new MessageResult<>();
        try {
            List<DeviceInfo> list = new ArrayList<>();
            //为公司管理员
            boolean is_admin = userService.hasRole(subject,"ADMIN");
            if(is_admin){
                User user = userService.get(subject);
                String companyId = user.getCompanyId();

                List<DeviceInfo> deviceInfoList = deviceInfoService.findByCompanyId(companyId);
                List<String> deviceIdList = new ArrayList<String>();
                if(deviceInfoList.size() > 0) {
                    for (DeviceInfo deviceInfo : deviceInfoList) {
                        deviceIdList.add(deviceInfo.getId());
                    }
                }else{
                    deviceIdList.add("-1");
                }
                list = deviceInfoService.findByDeviceIdList(deviceIdList);
            }
            else{
                list = deviceInfoService.list();
            }

            for (DeviceInfo deviceInfo:list) {
                String typeN = dataDictionaryService.findNameByCatalogNameAndValue("设备类型",deviceInfo.getType());
                deviceInfo.setTypeN(typeN);
            }

            msgResult.setResult(true);
            msgResult.setData(list);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }
        return msgResult;
    }

    @ApiOperation(value="获取设备验证规则列表")
    @PostMapping("getDeviceRule")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceId",value = "设备编号", paramType = "query")
    })
    public MessageResult<Map<String,Object>> getDeviceRule(String deviceId,@RequestAttribute String subject) {
        MessageResult<Map<String,Object>> msgResult = new MessageResult<>();
        try {
            Map<String,Object> ruleMap = lapiService.getDeviceRule(deviceId);
            ruleMap.get("RuleList");//规则状态列表
            //Mode 0安全帽,1口罩,2测温
            //Enabled 是否启用 0否,1是
            ruleMap.get("TemperatureRule");//温度设置
            //Type -> 1 手腕1,额头0
            //Minimum -> 30
            //Maximum -> 45
            //PreAlarmOffset -> 0.300000
            //AlarmThreshold -> 37.3000000
            msgResult.setResult(true);
            msgResult.setData(ruleMap);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }
        return msgResult;
    }

    @ApiOperation(value="更新设备验证规则")
    @PostMapping("saveDeviceRule")
    /**
    * @param deviceInfoId 设备id
     * @param needCap 安全帽验证 （0：不验证，1：验证）
    * @param needMask 口罩验证 （0：不验证，1：验证）
    * @param measureTemperature 测试验证 （0：不验证，1：验证）
    * @param type 测试验证类型 （0：测额温，1：测腕温）
    * @param minTemper 温度下限
     * @param maxTemper 温度上限
     * @param alarmThreshold 报警温度
     **/
    public MessageResult<DeviceInfo> saveDeviceRule(
            String deviceInfoId,
            Integer needCap,
            Integer needMask,
            Integer measureTemperature,
            Integer type,
            BigDecimal minTemper,
            BigDecimal maxTemper,
            BigDecimal alarmThreshold,
            @RequestAttribute String subject){
        MessageResult<DeviceInfo> msgResult = new MessageResult<>();
        Boolean result = false;
        try {
            result = lapiService.setDeviceRule(deviceInfoId,needCap,needMask,measureTemperature,
                    type,minTemper,maxTemper,alarmThreshold);
            msgResult.setResult(result);
            msgResult.setMessage("");
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }



    @ApiOperation(value = "查询")
    @RequestMapping(value = "query", method = RequestMethod.POST)
    public MessageResult<List> query(
            String aliasName,
            @RequestParam(value = "limit", defaultValue = "20") int limit,
            @RequestAttribute String subject) {
        MessageResult<List> msgResult = new MessageResult<>();

        Map<String, Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.is_online","desc"));
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(aliasName)) {
            searchParams.put("aliasName", "%" + aliasName + "%");
        }


        //为公司管理员
        boolean is_admin = userService.hasRole(subject,"ADMIN");
        if(is_admin){
            User user = userService.get(subject);
            String companyId = user.getCompanyId();
            searchParams.put("bindCompanyId", companyId);

        }

        List<DeviceInfo> deviceList = deviceInfoService.pageSearch(searchParams,1, limit,false, sortList);

        msgResult.setResult(true);
        msgResult.setData(deviceList);

        return msgResult;
    }

    @ResponseBody
    @ApiOperation(value="远程重启")
    @PostMapping("reboot")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceId",value = "设备编号", required = true,paramType = "query")
    })
    public MessageResult<String> reboot(String deviceId, @RequestAttribute String subject, HttpServletRequest request){
        MessageResult<String> msgResult = new MessageResult<>();

        SysLog sysLog = new SysLog();
        sysLog.setPointcut("远程重启设备");
        sysLog.setUserId(subject);
        sysLog.setData(deviceId);
        sysLog.setCreateTime(new Date());
        sysLog.setRemoteIp(HttpUtil.getIpAddress(request));

        try {
            boolean result = lapiService.reBoot(deviceId);

            msgResult.setResult(result);

            if (msgResult.isResult()){
                msgResult.setMessage("远程重启成功!");
            }
            else{
                msgResult.setMessage("远程重启失败!");
            }

            sysLog.setRemark(msgResult.getMessage());
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        sysLogService.insert(sysLog);

        return msgResult;
    }

    @ResponseBody
    @ApiOperation(value="设备连接列表")
    @GetMapping("channelList")
    public MessageResult<List> channelList(){
        MessageResult<List> msgResult = new MessageResult<>();

        try {
            List<Map> mapList = new ArrayList<>();

            Set<String> set = LapiSessionUtil.keySet();

            for (String key : set) {
                Map<String,Object> map = new HashMap<>();
                map.put("deviceNo",key);

                //所属单位
                DeviceInfo deviceInfo = deviceInfoService.getByDeviceNo(key);

                if(deviceInfo!=null){
                    //设备别名
                    map.put("aliasName", deviceInfo.getAliasName());

                    CompanyInfo companyInfo = companyInfoService.get(deviceInfo.getCompanyId());

                    if(companyInfo!=null){
                        map.put("companyName", companyInfo.getName());
                    }
                }

                LapiChannel lapiChannel = LapiSessionUtil.get(key);

                if (lapiChannel!=null){
                    map.put("remoteAddress",lapiChannel.getChannel().remoteAddress().toString());
                    map.put("isOpen",lapiChannel.getChannel().isOpen());
                }

                mapList.add(map);
            }

            mapList.sort((Map o1, Map o2)->{
                try {
                    String companyName1 = (String) o1.get("companyName");
                    String companyName2 = (String) o2.get("companyName");

                    String deviceNo1 = (String) o1.get("deviceNo");
                    String deviceNo2 = (String) o2.get("deviceNo");

                    int result1 = companyName1.compareTo(companyName2);

                    if (result1 != 0) {
                        return result1;
                    } else {
                        return deviceNo1.compareTo(deviceNo2);
                    }
                }
                catch (Exception ex){
                    logger.error(ex.getMessage(),ex);
                    return -1;
                }
            });

            //增加序号
            for (int i=0;i<mapList.size();i++){
                mapList.get(i).put("index", i+1);
            }

            msgResult.setData(mapList);
            msgResult.setResult(true);
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
            ex.printStackTrace();
        }

        return msgResult;
    }


    @ResponseBody
    @ApiOperation(value="批量修改设备IP和端口")
    @PostMapping("updateDeviceData")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceNos",value = "设备编号", required = true,paramType = "query"),
            @ApiImplicitParam(name = "newAddress",value = "新IP", required = true,paramType = "query"),
            @ApiImplicitParam(name = "newPort",value = "新端口", required = true,paramType = "query")
    })
    public MessageResult<String> updateDeviceData(
            String deviceNos,
            String newAddress,
            String newPort,
            @RequestAttribute String subject,
            HttpServletRequest request){
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            List<String> deviceNoList = null;
            if(StringUtils.isEmpty(deviceNos)){
                throw new Exception("未选择修改设备");
            }else{
                deviceNoList = Arrays.asList(deviceNos.split(","));
            }

            if(StringUtils.isEmpty(newAddress)){
                throw new Exception("新IP不能为空");
            }
            if(StringUtils.isEmpty(newPort)){
                throw new Exception("新端口不能为空");
            }
            lapiService.updateDeviceData(deviceNoList,newAddress,newPort);
            msgResult.setResult(true);
            msgResult.setMessage("修改成功!");

        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }
        return msgResult;
    }

}
