package com.jpsoft.smart.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.smart.modules.base.entity.HolidayInfo;
import com.jpsoft.smart.modules.base.entity.CompanyInfo;
import com.jpsoft.smart.modules.base.service.HolidayInfoService;
import com.jpsoft.smart.modules.base.service.CompanyInfoService;
import com.jpsoft.smart.modules.common.dto.MessageResult;
import com.jpsoft.smart.modules.common.dto.Sort;
import com.jpsoft.smart.modules.common.utils.PojoUtils;
import com.jpsoft.smart.modules.sys.entity.User;
import com.jpsoft.smart.modules.sys.service.UserService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 节假日设置
 */
@RestController
@RequestMapping("/base/holidayInfo")
public class HolidayInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private HolidayInfoService holidayInfoService;

    @Autowired
    private UserService userService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<HolidayInfo> create(){
        MessageResult<HolidayInfo> msgResult = new MessageResult<>();

        HolidayInfo holidayInfo = new HolidayInfo();

        msgResult.setData(holidayInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<HolidayInfo> add(@RequestBody HolidayInfo holidayInfo,@RequestAttribute String subject){
        MessageResult<HolidayInfo> msgResult = new MessageResult<>();

        try {
            holidayInfo.setId(UUID.randomUUID().toString());
            holidayInfo.setDelFlag(false);
            holidayInfo.setCreateBy(subject);
            holidayInfo.setCreateTime(new Date());

            int affectCount = holidayInfoService.insert(holidayInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(holidayInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<HolidayInfo> edit(@PathVariable("id") String id){
        MessageResult<HolidayInfo> msgResult = new MessageResult<>();

        try {
            HolidayInfo holidayInfo = holidayInfoService.get(id);

            if (holidayInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(holidayInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<HolidayInfo> update(@RequestBody HolidayInfo holidayInfo,@RequestAttribute String subject){
        MessageResult<HolidayInfo> msgResult = new MessageResult<>();

        try {
            holidayInfo.setUpdateBy(subject);
            holidayInfo.setUpdateTime(new Date());

            int affectCount = holidayInfoService.update(holidayInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(holidayInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            HolidayInfo holidayInfo = holidayInfoService.get(id);
            holidayInfo.setDelFlag(true);
            holidayInfo.setUpdateBy(subject);
            holidayInfo.setUpdateTime(new Date());

            int affectCount = holidayInfoService.update(holidayInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                HolidayInfo holidayInfo = holidayInfoService.get(id);
                holidayInfo.setDelFlag(true);
                holidayInfo.setUpdateBy(subject);
                holidayInfo.setUpdateTime(new Date());

                affectCount += holidayInfoService.update(holidayInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("begin_date","asc"));

        Page<HolidayInfo> page = holidayInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
